@@ -7,7 +7,8 @@ A set of Python decorators to ease the development of AWS lambda functions.
 import json
 from http import HTTPStatus
 import boto3
-from aws_lambda_decorators.utils import full_name, all_func_args, find_key_case_insensitive, failure, get_logger
+from aws_lambda_decorators.utils import (full_name, all_func_args, find_key_case_insensitive, failure, get_logger,
+                                         find_websocket_connection_id, get_websocket_endpoint)
 
 
 LOGGER = get_logger(__name__)
@@ -24,9 +25,10 @@ CORS_INVALID_TYPE_ERROR = "Invalid value type in CORS header"
 CORS_NON_DICT_ERROR = "Invalid response type for CORS headers"
 CORS_INVALID_TYPE_LOG_MESSAGE = "Cannot set %s header to a non %s value"
 CORS_NON_DICT_LOG_MESSAGE = "Cannot add headers to a non dictionary response"
+UNKNOWN = "Unknown"
 
 
-def extract_from_event(parameters, group_errors=False):
+def extract_from_event(parameters, group_errors=False, allow_none_defaults=False):
     """
     Extracts a set of parameters from the event dictionary in a lambda handler.
 
@@ -44,10 +46,10 @@ def extract_from_event(parameters, group_errors=False):
     """
     for param in parameters:
         param.func_param_name = "event"
-    return extract(parameters, group_errors)
+    return extract(parameters, group_errors, allow_none_defaults)
 
 
-def extract_from_context(parameters, group_errors=False):
+def extract_from_context(parameters, group_errors=False, allow_none_defaults=False):
     """
     Extracts a set of parameters from the context dictionary in a lambda handler.
 
@@ -65,10 +67,10 @@ def extract_from_context(parameters, group_errors=False):
     """
     for param in parameters:
         param.func_param_name = "context"
-    return extract(parameters, group_errors)
+    return extract(parameters, group_errors, allow_none_defaults)
 
 
-def extract(parameters, group_errors=False):
+def extract(parameters, group_errors=False, allow_none_defaults=False):
     """
     Extracts a set of parameters from any function parameter passed to an AWS lambda handler.
 
@@ -83,10 +85,14 @@ def extract(parameters, group_errors=False):
         parameters (list): A collection of Parameter type items.
         group_errors (bool): flag that indicates if error messages are to be grouped together
             (if set to False, validation will end on first error)
+        allow_none_defaults: A flag to allow None defaults. If True, None defaults will be passed into the kwargs.
+            If the flag is set to False, the None defaults will not be added to kwargs, and the default will be
+            picked up (if exists) from the method signature.
     """
     def decorator(func):
         def wrapper(*args, **kwargs):
             try:
+                param = None
                 errors = []
                 arg_dictionary = all_func_args(func, args, kwargs)
                 for param in parameters:
@@ -98,17 +104,19 @@ def extract(parameters, group_errors=False):
                         if not group_errors:
                             LOGGER.error(VALIDATE_ERROR_MESSAGE, errors)
                             return failure(errors)
-                    elif return_val is not None:
+                    elif allow_none_defaults or return_val is not None:
                         kwargs[param.get_var_name()] = return_val
             except Exception as ex:  # noqa: pylint - broad-except
-                LOGGER.error(EXCEPTION_LOG_MESSAGE, full_name(ex), str(ex), param.func_param_name, param.path)
+                LOGGER.error(EXCEPTION_LOG_MESSAGE, full_name(ex), str(ex),
+                             param.func_param_name if param else UNKNOWN,
+                             param.path if param else UNKNOWN)
                 return failure(ERROR_MESSAGE)
+            else:
+                if group_errors and errors:
+                    LOGGER.error(VALIDATE_ERROR_MESSAGE, errors)
+                    return failure(errors)
 
-            if group_errors and errors:
-                LOGGER.error(VALIDATE_ERROR_MESSAGE, errors)
-                return failure(errors)
-
-            return func(*args, **kwargs)
+                return func(*args, **kwargs)
         return wrapper
     return decorator
 
@@ -129,8 +137,8 @@ def handle_exceptions(handlers):
         def wrapper(*args, **kwargs):
             try:
                 return func(*args, **kwargs)
-            except tuple([handler.exception for handler in handlers]) as ex:  # noqa: pylint - catching-non-exception
-                failed_handler = [handler for handler in handlers if handler.exception is type(ex)][0]
+            except tuple(handler.exception for handler in handlers) as ex:  # noqa: pylint - catching-non-exception
+                failed_handler = [handler for handler in handlers if isinstance(ex, handler.exception)][0]
                 message = failed_handler.friendly_message
 
                 if message and str(ex):
@@ -327,3 +335,87 @@ def cors(allow_origin=None, allow_methods=None, allow_headers=None, max_age=None
                 return failure(CORS_NON_DICT_ERROR, HTTPStatus.INTERNAL_SERVER_ERROR)
         return wrapper
     return decorator
+
+
+def push_ws_errors(websocket_endpoint_url: str):
+    """
+    Handles and pushes any unsuccessful responses as errors to the calling client via websockets
+
+    Usage:
+        @push_ws_errors('https://api_id.execute_id.region.amazonaws.com/Prod')
+        @handle_all_exceptions()
+        def handler(event, context):
+            return {
+                'statusCode': 400,
+                'body': {
+                    'message': 'Bad request'
+                }
+            }
+
+    Args:
+        websocket_endpoint_url (str): The api gateway connection URL
+
+    Returns:
+        the original response from the lambda handler
+    """
+    def decorator(func):
+        def wrapper(*args, **kwargs):
+            connection_id = find_websocket_connection_id(args)
+
+            response = func(*args, **kwargs)
+            success = response.get("statusCode", HTTPStatus.INTERNAL_SERVER_ERROR).value < 300
+
+            if connection_id and not success:
+                websocket_endpoint = get_websocket_endpoint(websocket_endpoint_url)
+
+                ws_response = {
+                    "type": "error",
+                    "statusCode": response.get("statusCode", HTTPStatus.INTERNAL_SERVER_ERROR),
+                    "message": json.loads(response.get("body", "{}")).get("message")
+                }
+
+                websocket_endpoint.post_to_connection(
+                    ConnectionId=connection_id,
+                    Data=json.dumps(ws_response)
+                )
+
+            return response
+        return wrapper
+    return decorator
+
+
+def push_ws_response(websocket_endpoint_url: str):
+    """
+    Handles and pushes all responses to the calling client via websockets
+
+    Usage:
+        @push_ws_response('https://api_id.execute_id.region.amazonaws.com/Prod')
+        def handler(event, context):
+            return {
+                'statusCode': 200,
+                'body': 'Hello, world!'
+            }
+
+    Args:
+        websocket_endpoint_url (str): The api gateway connection URL
+
+    Returns:
+        the original response from the lambda handler
+    """
+    def decorator(func):
+        def wrapper(*args, **kwargs):
+            connection_id = find_websocket_connection_id(args)
+
+            response = func(*args, **kwargs)
+
+            if connection_id:
+                websocket_endpoint = get_websocket_endpoint(websocket_endpoint_url)
+
+                websocket_endpoint.post_to_connection(
+                    ConnectionId=connection_id,
+                    Data=json.dumps(response)
+                )
+
+            return response
+        return wrapper
+    return decorator
